// Con Clase Febrero 2024
// Ejemplo de documento XML nuevo
// Añadir en las opciones de compilador:
// xml2.dll para usar la versión dinámica de libxml2

#include <iostream>
#include <libxml/xpath.h>
#include <cstring>

int main() {
    xmlDocPtr doc;
    xmlNodePtr nodo_raiz, nodoNombres, nodoCiudades, nodo;
    xmlNodePtr comentario;
    const xmlChar *version = BAD_CAST "1.0";
    xmlChar documento[] = "aplicacion";
    xmlChar nombre[] = "nombres";
    xmlChar ciudad[] = "ciudades";
    xmlChar valor[] = "valor";
    xmlChar defecto[] = "pordefecto";
    const char *nombres[] = {
        "Carlos",
        "Antonio",
        "José",
        "Clara",
        "María",
        "Fernando",
        "Eva"
    };
    size_t nombre_defecto = 3;
    const char *ciudades[] = {
        "Málaga",
        "Jaén",
        "Badalona",
        "Vigo",
        "Londres",
        "Berlín"
    };
    size_t ciudad_defecto = 1;

    LIBXML_TEST_VERSION;

    // Crear el documento:
    doc = xmlNewDoc(version);
    xmlNewDocProp(doc, BAD_CAST "prop", BAD_CAST "viejo");

    // Insertar un comentario para el documento:
    comentario = xmlNewDocComment(doc, BAD_CAST "Historial de valores");
    xmlAddChild(xmlNodePtr(doc), comentario);

    // Crear el nodo principal:
    nodo_raiz = xmlNewDocNode(doc, NULL, documento, NULL);
    xmlDocSetRootElement(doc, nodo_raiz);

    // Crear un nodo para los nombres:
    nodoNombres = xmlNewChild(nodo_raiz, NULL, nombre, NULL);

    for(size_t i = 0; i < sizeof(nombres)/sizeof(nombres[0]); i++) {
        nodo = xmlNewChild(nodoNombres, NULL, valor, BAD_CAST nombres[i]);
        if(i == nombre_defecto) {
            xmlNewProp(nodo, defecto, BAD_CAST "Si");
        }
    }

    nodoCiudades = xmlNewChild(nodo_raiz, NULL, ciudad, NULL);
    for(size_t i = 0; i < sizeof(ciudades)/sizeof(ciudades[0]); i++) {
        nodo = xmlNewChild(nodoCiudades, NULL, valor, BAD_CAST ciudades[i]);
        if(i == ciudad_defecto) {
            xmlNewProp(nodo, defecto, BAD_CAST "Si");
        }
    }

    xmlNodePtr n = xmlNewDocNode(doc, NULL, BAD_CAST "Valor1", BAD_CAST "Un valor X");
    xmlNodePtr m = xmlAddChild(nodo_raiz, n);
    n = xmlNewDocNode(doc, NULL, BAD_CAST "Valor2", BAD_CAST "Un valor Y");
    xmlAddSibling(m, n);
    n = xmlNewDocNode(doc, NULL, BAD_CAST "Valor0", BAD_CAST "Un valor Z");
    xmlAddPrevSibling(m, n);

    xmlNewTextChild(nodo_raiz, NULL, BAD_CAST "texto", BAD_CAST "Uno nodo que contiene <texto>");

    // Nodo de texto:
    xmlNodePtr texto = xmlNewDocText(doc, BAD_CAST "Esto es un nodo de texto.");
    xmlAddChild(nodo_raiz, texto);

    // Mostrar en pantalla:
    xmlSaveFileEnc("-", doc, "UTF8");
    // Guardar en fichero:
    xmlSaveFileEnc("opciones.xml", doc, "UTF8");

    // Guardar en memoria:
    xmlChar *mem;
    int tam;
    xmlDocDumpMemoryEnc(doc, &mem, &tam, "UTF8");
    std::cout << mem << std::endl;
    xmlFree(mem);

    xmlFreeDoc(doc);
    xmlMemoryDump();
    return 0;
}
